@@ -1,21 +1,15 @@
-import asyncio
 from pathlib import Path
 from typing import List, cast
 
 import click
 from nb_cli import _
 from noneprompt import Choice, ListPrompt, CancelledError
+from nb_cli.handlers import detect_virtualenv, get_python_version
 from nb_cli.cli import CLI_DEFAULT_STYLE, ClickAliasedGroup, run_sync, run_async
-from nb_cli.handlers import (
-    detect_virtualenv,
-    terminate_process,
-    get_python_version,
-    remove_signal_handler,
-    register_signal_handler,
-)
 
-from .utils import safe_write_file
+from .utils import safe_copy_dir, safe_write_file
 from .handler import (
+    compose_ps,
     compose_up,
     compose_down,
     compose_logs,
@@ -75,8 +69,9 @@ async def docker(ctx: click.Context):
     default=False,
     help="Force to re-generate the Dockerfile.",
 )
+@click.pass_context
 @run_async
-async def generate(cwd: Path, venv: bool, force: bool):
+async def generate(ctx: click.Context, cwd: Path, venv: bool, force: bool):
     """Generate Dockerfile and docker-compose.yml."""
     if python_path := detect_virtualenv() if venv else None:
         click.secho(
@@ -86,26 +81,36 @@ async def generate(cwd: Path, venv: bool, force: bool):
             fg="green",
         )
 
+    if not cwd.is_dir():
+        click.secho(f"Directory {cwd} does not exist.", fg="red")
+        ctx.exit(1)
+
     python_version = await get_python_version(python_path)
     python_version = f"{python_version['major']}.{python_version['minor']}"
     is_reverse = await get_driver_type(python_path=python_path, cwd=cwd)
     build_backend = await get_build_backend()
 
-    dockerfile = await generate_dockerfile(
-        python_version=python_version,
-        is_reverse=is_reverse,
-        build_backend=build_backend,
-    )
-    await safe_write_file(cwd / "Dockerfile", dockerfile, force=force)
+    try:
+        dockerfile = await generate_dockerfile(
+            python_version=python_version,
+            is_reverse=is_reverse,
+            build_backend=build_backend,
+        )
+        await safe_write_file(cwd / "Dockerfile", dockerfile, force=force)
 
-    compose_file = await generate_compose_file(is_reverse=is_reverse)
-    await safe_write_file(cwd / "docker-compose.yml", compose_file, force=force)
+        compose_file = await generate_compose_file(is_reverse=is_reverse)
+        await safe_write_file(cwd / "docker-compose.yml", compose_file, force=force)
 
-    if not is_reverse:
-        return
+        await safe_copy_dir(
+            Path(__file__).parent / "static" / "common", cwd, force=force
+        )
 
-    for file in (Path(__file__).parent / "docker").iterdir():
-        await safe_write_file(cwd / "docker" / file.name, file.read_text(), force=force)
+        if is_reverse:
+            await safe_copy_dir(
+                Path(__file__).parent / "static" / "reverse", cwd, force=force
+            )
+    except CancelledError:
+        ctx.exit()
 
 
 @docker.command(aliases=["run"], context_settings={"ignore_unknown_options": True})
@@ -137,22 +142,8 @@ async def up(
     ):
         await run_sync(ctx.invoke)(generate)
 
-    should_exit = asyncio.Event()
-
-    def shutdown(signum, frame):
-        should_exit.set()
-
-    async def wait_for_shutdown():
-        await should_exit.wait()
-        await terminate_process(proc)
-
-    register_signal_handler(shutdown)
     proc = await compose_up(compose_args, cwd=cwd)
-    task = asyncio.create_task(wait_for_shutdown())
     await proc.wait()
-    should_exit.set()
-    await task
-    remove_signal_handler(shutdown)
 
 
 @docker.command(aliases=["stop"], context_settings={"ignore_unknown_options": True})
@@ -161,22 +152,8 @@ async def up(
 @run_async
 async def down(cwd: Path, compose_args: List[str]):
     """Undeploy the bot."""
-    should_exit = asyncio.Event()
-
-    def shutdown(signum, frame):
-        should_exit.set()
-
-    async def wait_for_shutdown():
-        await should_exit.wait()
-        await terminate_process(proc)
-
-    register_signal_handler(shutdown)
     proc = await compose_down(compose_args, cwd=cwd)
-    task = asyncio.create_task(wait_for_shutdown())
     await proc.wait()
-    should_exit.set()
-    await task
-    remove_signal_handler(shutdown)
 
 
 @docker.command(context_settings={"ignore_unknown_options": True})
@@ -185,22 +162,8 @@ async def down(cwd: Path, compose_args: List[str]):
 @run_async
 async def build(cwd: Path, compose_args: List[str]):
     """Build the bot image."""
-    should_exit = asyncio.Event()
-
-    def shutdown(signum, frame):
-        should_exit.set()
-
-    async def wait_for_shutdown():
-        await should_exit.wait()
-        await terminate_process(proc)
-
-    register_signal_handler(shutdown)
     proc = await compose_build(compose_args, cwd=cwd)
-    task = asyncio.create_task(wait_for_shutdown())
     await proc.wait()
-    should_exit.set()
-    await task
-    remove_signal_handler(shutdown)
 
 
 @docker.command(context_settings={"ignore_unknown_options": True})
@@ -209,19 +172,15 @@ async def build(cwd: Path, compose_args: List[str]):
 @run_async
 async def logs(cwd: Path, compose_args: List[str]):
     """View the bot logs."""
-    should_exit = asyncio.Event()
-
-    def shutdown(signum, frame):
-        should_exit.set()
-
-    async def wait_for_shutdown():
-        await should_exit.wait()
-        await terminate_process(proc)
-
-    register_signal_handler(shutdown)
     proc = await compose_logs(compose_args, cwd=cwd)
-    task = asyncio.create_task(wait_for_shutdown())
     await proc.wait()
-    should_exit.set()
-    await task
-    remove_signal_handler(shutdown)
+
+
+@docker.command(context_settings={"ignore_unknown_options": True})
+@click.option("-d", "--cwd", default=".", type=Path, help="The working directory.")
+@click.argument("compose_args", nargs=-1)
+@run_async
+async def ps(cwd: Path, compose_args: List[str]):
+    """View the bot service status."""
+    proc = await compose_ps(compose_args, cwd=cwd)
+    await proc.wait()
