@@ -8,4 +8,4 @@ from .cli import docker
 def install():
     cli_ = cast(CLIMainGroup, cli)
     cli_.add_command(docker)
-    cli_.add_aliases("docker", ["deploy"])
+    cli_.add_aliases("docker", ["deploy", "compose"])
